#Copyright 2001 by R. David Murray and FCGNetworks (http://www.fcgnetworks.net)
#Governed by the Artistic License
#  http://www.opensource.org/licenses/artistic-license.html

#For version 0.0.1

#Note: should add tests for case of non-seekable file

from unittest import TestCase, makeSuite, TextTestRunner
from xfile import xfile
from string import split, find

class FakeFile:

  def __init__(self,l,missingnewline=0):
    self.data = l
    self.pointer = 0
    self.missingnewline = missingnewline
  def readline(self):
    if self.pointer<0 or self.pointer>=len(self.data): return ''
    self.pointer = self.pointer + 1
    if self.missingnewline and self.pointer==len(self.data):
      return self.data[self.pointer-1]
    return self.data[self.pointer-1]+'\n'
  def tell(self): return self.pointer
  def seek(self,i): self.pointer = i

class TestCases(TestCase):

  def setUp(self):
    self.testlist = [
      'line one:test',
      'line two',
      '',
      '',
      'line five:test',
    ]
    self.fakefile = FakeFile(self.testlist[:])
    self.xfile = xfile(FakeFile(self.testlist[:]))
    self.xfiletokens = xfile(FakeFile(self.testlist[:]),parser=split)
    self.xfilecolon = xfile(FakeFile(self.testlist[:]),
      lambda x: find(x,':')>=0,lambda x: split(x[:-1],':'))
    self.xfilenonl = xfile(FakeFile(self.testlist[:],missingnewline=1))

  def assertEQ(self,res,correct,msg=''):
    assert res==correct, "%s <> %s %s" % (repr(res),repr(correct),msg)

  def testFor(self):
    res = []
    for line in self.xfile: res.append(line)
    self.assertEQ(res,self.testlist)

  def testSeq(self):
    f = self.xfile
    self.assertEQ(f[0],self.testlist[0])
    self.assertEQ(f[1],self.testlist[1])
    self.assertEQ(f[1],self.testlist[1],'on second reference')
    self.assertEQ(f[0],self.testlist[0])
    self.assertEQ(f[1],self.testlist[1],'after 0 read')
    self.assertEQ(f[0],self.testlist[0],'another rewind')
    self.assertEQ(f[3],self.testlist[3])
    self.assertEQ(f[0],self.testlist[0],'after 3 read')
    self.assertEQ(f[4],self.testlist[4])
    self.assertRaises(IndexError,self.xfile.__getitem__,5)
    self.assertRaises(IndexError,self.xfile.__getitem__,7)

  def testParser(self):
    f = self.xfiletokens
    for i in range(len(self.testlist)):
      self.assertEQ(f[i],split(self.testlist[i]),'(item %s)' % i)

  def testRaw(self):
    f = self.xfile
    for i in range(len(self.testlist)):
      f[i]
      self.assertEQ(f.raw(),self.testlist[i]+'\n','(item %s)' % i)

  def testSetParser(self):
    f = self.xfile
    self.assertEQ(f[0],self.testlist[0])
    f.setParser(lambda x: split(x[:-1],':'))
    self.assertEQ(f[0],split(self.testlist[0],':'))
    self.assertEQ(f[1],split(self.testlist[1],':'))

  def testLineno(self):
    f = self.xfile
    for i in range(len(self.testlist)):
      f[i]
      self.assertEQ(f.lineno(),i+1)
    f[0]
    self.assertEQ(f.lineno(),1)

  def testNext(self):
    f = self.xfile
    for i in range(len(self.testlist)):
      self.assertEQ(f.next(),self.testlist[i])
    f[2]
    self.assertEQ(f.next(),self.testlist[3])

  def testNextnonblank(self):
    f = self.xfile
    self.assertEQ(f.nextnonblank(),self.testlist[0])
    self.assertEQ(f.nextnonblank(),self.testlist[1])
    self.assertEQ(f.nextnonblank(),self.testlist[4])

  def testNextblank(self):
    f = self.xfile
    self.assertEQ(f.nextblank(),'')
    self.assertEQ(f.next(),self.testlist[3])

  def testSelector(self):
    f = self.xfilecolon
    self.assertEQ(f[0],split(self.testlist[0],':'))
    self.assertEQ(f[1],split(self.testlist[4],':'))
    self.assertRaises(IndexError,f.__getitem__,2)

  def testLen(self): self.assertEQ(len(self.xfile),5)

  def testNegativeIndex(self):
    f = self.xfile
    self.assertEQ(f[-1],self.testlist[-1])
    self.assertEQ(f[-3],self.testlist[-3])
    self.assertEQ(f[3],self.testlist[3])

  def testSeek(self):
    f = self.xfile
    self.assertRaises(IndexError,f.seek,-3)
    self.assertRaises(IndexError,f.seek,5)
    len(f)
    self.assertRaises(IndexError,f.seek,5)

  def testTell(self):
    f = self.xfile
    for i in range(len(self.testlist)):
      self.assertEQ(f.tell(),i)
      f[i]
      self.assertEQ(f.tell(),i+1)

  def testContains(self):
    f = self.xfile
    self.assertEQ('' in f,1)
    self.assertEQ('line two' in f,1)
    self.assertEQ('line six' in f,0)

  def testSlice(self):
    f = self.xfile
    self.assertEQ(f[1:3],self.testlist[1:3])
    self.assertEQ(f[-2:-1],self.testlist[-2:-1])
    self.assertEQ(f[2:2],self.testlist[2:2])
    self.assertEQ(f[6:7],self.testlist[6:7])
    self.assertEQ(f[:],self.testlist[:])

  def testAdd(self):
    f = self.xfile
    self.assertEQ(f+['test'],self.testlist+['test'])
    self.assertEQ(['test']+f,['test']+self.testlist)

  def testCount(self):
    f = self.xfile
    self.assertEQ(f.count(self.testlist[1]),1)
    self.assertEQ(f.count(''),2)

  def testIndex(self):
    f = self.xfile
    self.assertEQ(f.index(self.testlist[1]),1)
    self.assertEQ(f.index(''),2)
    self.assertRaises(ValueError,f.index,'notthereatall')

  def testMissingNewLine(self):
    f = self.xfilenonl
    self.assertEQ(f[2],self.testlist[2])
    self.assertEQ(f[4],self.testlist[4])
    self.assertRaises(IndexError,f.__getitem__,5)
    f[3]
    self.assertEQ(f.nextnonblank(),self.testlist[4])

suite = makeSuite(TestCases,'test')

if __name__ == '__main__':
  from sys import exit
  if not TextTestRunner().run(suite).wasSuccessful(): exit(1)
